/*
	Name: usb_descriptors.c
	Date: 1.Mar.2013
	Encoding: UTF-8
	Author: Michal Riša <xrisam01@stud.fit.vutbr.cz>
	Licence:
		Copyright (C) 2013 Michal Riša <mh74s74@gmail.com>

		This file is part of bp_xrisam01.

		bp_xrisam01 is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License version 3 as
		published by the Free Software Foundation.

		bp_xrisam01 is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with bp_xrisam01. If not, see <http://www.gnu.org/licenses/>.
*/

#include <hidef.h>
#include "types.h"
#include "usb_drv.h"
#include "usb.h"

/* vsetky viacbajtove polozky su poskladane v little endian byte order */

//------------------------------------------------------------------------------
//device descriptor
#define DEVICE_DESCRIPTOR_BYTES (18)

const uint8_t device_descriptor[DEVICE_DESCRIPTOR_BYTES] = {
	0x12, //bLength (DEVICE_DESCRIPTOR_BYTES)
	0x01, //bDescriptorType
	0x00, 0x02, //bcdUSB, 0x0200
	0x00, //bDeviceClass
	0x00, //bDeviceSubClass
	0x00, //bDeviceProtocol
	USB_EP0_BUF_SIZE, //bMaxPacketSize
	0x6d, 0x04, //idVendor 0x046d
	0x16, 0xc0, //idProduct 0xc016
	0x40, 0x03, //bcdDevice 0x0340
	0x01, //iManufacturer = "mh74s74/Magorsoft"
	0x02, //iProduct = "Mouse"
	0x03, //iSerialNumber = "001"
	0x01, //bNumConfigurations
};
//------------------------------------------------------------------------------
//configuration descriptor
const uint8_t cfg0[] = {
/* configuration_descriptor */
	0x09, //bLength (CONFIGURATION_DESCRIPTOR_BYTES)
	0x02, //bDescriptorType
	0x22, 0x00, //wTotalLength 0x0022 (34B)
	0x01, //bNumInterfaces
	0x01, //bConfigurationValue
	0x04, //iConfiguration = "cfg0"
	0b10100000, //bmAttributes
	0x32, //bMaxPower
/* interface_descriptor */
	0x09, //bLength (INTERFACE_DESCRIPTOR_BYTES)
	0x04, //bDescriptorType
	0x00, //bInterfaceNumber
	0x00, //bAlternateSetting
	0x01, //bNumEndpoints
	0x03, //bInterfaceClass
	0x01, //bInterfaceSubClass
	0x02, //bInterfaceProtocol
	0x05, //iInterface = "iface0"
/* hid_descriptor */
	0x09, //bLength
	0x21, //bDescriptorType
	0x10, 0x01, //bcdHID [2B] 0x0110
	0x00, //bCountryCode
	0x01, //bNumDescriptors
	0x22, //bDescriptorType
	0x34, 0x00, //wDescriptorLength [2B] 0x0034 [52B, sizeof hid_report]
/* endpoint_descriptor */
	0x07, //bLength (ENDPOINT_DESCRIPTOR_BYTES)
	0x05, //bDescriptorType
	0x81, //bEndpointAddress
	0x03, //bmAttributes
	0x04, 0x00, //wMaxPacketSize [2B] 0x0004
	0x0A, //bInterval 10ms
};

const uint8_t *cfg_desc_array[] = { /* pole konfiguracnych deskriptorov */
	cfg0,
};
//------------------------------------------------------------------------------
//obsluzne funkcie konfiguracii
#define CFG_HDR_CNT /* pocet konfiguracii zariadenia */\
	(sizeof cfg_desc_array / sizeof cfg_desc_array[0])

//prototyp obsluznej funkcie konfiguracie
extern void mouse_cfg(uint8_t event);

//pole obsluznych funkcii konfiguracii
const void (*cfg_hdrs[CFG_HDR_CNT])(uint8_t event) = {
	mouse_cfg,
};
//------------------------------------------------------------------------------
//alternativne nastavenia rozhrani
uint8_t alt_interface_array[] = {
	0,
};
//------------------------------------------------------------------------------
//stringove deskriptory
const uint8_t string_lang[] = {
	4, //bLength
	0x03, //bDescriptorType
	0x09, 0x04, //string
};

const uint8_t string_manufact[] = {
	30, //bLength
	0x03, //bDescriptorType
	'M', 0, 'h', 0, '7', 0, '4', 0, 's', 0, '7', 0, '4', 0,
	'/', 0, 'M', 0, 'h', 0, 's', 0, 'o', 0, 'f', 0, 't', 0, //string
};

const uint8_t string_product[] = {
	12, //bLength
	0x03, //bDescriptorType
	'M', 0, 'o', 0, 'u', 0, 's', 0, 'e', 0, //string
};

const uint8_t string_serial[] = {
	8, //bLength
	0x03, //bDescriptorType
	'0', 0, '0', 0, '1', 0, //string
};

const uint8_t string_cfg0[] = {
	10, //bLength
	0x03, //bDescriptorType
	'c', 0, 'f', 0, 'g', 0, '0', 0, //string
};

const uint8_t string_iface0[] = {
	14, //bLength
	0x03, //bDescriptorType
	'i', 0, 'f', 0, 'a', 0, 'c', 0, 'e', 0, '0', 0, //string
};

const uint8_t *str_desc_array[] = { /* pole stringovych deskriptorov */
	string_lang,
	string_manufact,
	string_product,
	string_serial,
	string_cfg0,
	string_iface0,
};

