/*
	Name:husb_drv.h
	Date: 16.Dec.2012
	Encoding: UTF-8
	Author: Michal Riša <xrisam01@stud.fit.vutbr.cz>
	Licence:
		Copyright (C) 2013 Michal Riša <mh74s74@gmail.com>

		This file is part of bp_xrisam01.

		bp_xrisam01 is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License version 3 as
		published by the Free Software Foundation.

		bp_xrisam01 is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with bp_xrisam01. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef USB_DRV_H
#define USB_DRV_H

#include "types.h" /* (u)intxx_t */

//------------------------------------------------------------------------------
//makra pre staticku inicializaciu self-power a remote-wakeup priznakov
//#define USB_SELF_POWERED
//#define USB_REMOTE_WAKEUP

//makro pre zapnutie VREG
#define VREG_EN
//------------------------------------------------------------------------------
//USB modul MC9S08JM60

//endpoint descriptor
struct usb_ep_descr {
	union {
		struct {
			unsigned :1;
			unsigned :1;
			unsigned BDTSTALL:1; /* TKPID */
			unsigned DTS:1; /* TKPID */
			unsigned :1; /* TKPID */
			unsigned :1; /* TKPID */
			unsigned DATA:1;
			unsigned OWN:1;
		}bits;
		uint8_t reg;
	}u;
	uint8_t bc; /* byte count */
	uint8_t epadr; /* adresa endpointu v USB RAM */
};

enum { /* priznaky do ep.u.r_bits */
	EPF_OWN_MCU = 0b00000000, /* CPU vlastni buffer */
	EPF_OWN_SIE = 0b10000000, /* SIE vlastni buffer */
	EPF_DATA0   = 0b00000000, /* DATA0 packet */
	EPF_DATA1   = 0b01000000, /* DATA1 packet */
	EPF_DTS     = 0b00001000,
	EPF_STALL   = 0b00000100,
};

//vypocet adresy buffra endpointu
#define EPADR(buffer) \
	(uint8_t)(((uint16_t)&buffer - (uint16_t)0x1860) >> 2)

//inicializacia buffra endpointu
#define USB_EP_INIT(ep_descr, flags, buffer) {\
	ep_descr.u.reg = flags;\
	ep_descr.bc = sizeof(buffer);\
	ep_descr.epadr = (uint8_t)(((uint16_t)&buffer - (uint16_t)0x1860) >> 2);\
}

#define USB_EP_PID(endpoint) \
	(endpoint.u.reg & 0b00111100)

//buffer descriptor table
struct usb_bdt { /* 32B */
	struct usb_ep_descr ep0in;
	struct usb_ep_descr ep0out;
	struct usb_ep_descr ep1;
	struct usb_ep_descr ep2;
	struct usb_ep_descr ep3;
	struct usb_ep_descr ep4;
	struct usb_ep_descr ep5even;
	struct usb_ep_descr ep5odd;
	struct usb_ep_descr ep6even;
	struct usb_ep_descr ep6odd;
	uint8_t _pad[2]; /* reserved */
};

//priznaky do EPCTLn(0-6) registrov
enum {
	EP_RX    = 0b00001000,
	EP_TX    = 0b00000100,
	EP_STALL = 0b00000010,
	EP_HSK   = 0b00000001,
};
//------------------------------------------------------------------------------
//usb_drv.c
enum {
	EVENT_ACTIVATE,
	EVENT_DEACTIVATE,
	EVENT_SUSPEND,
	EVENT_WAKEUP,
};

void usb_init(void);
void usb_clock_init(void);
uint8_t check_remote_wakeup(void);
//------------------------------------------------------------------------------
//obsluzne funkcie
extern void usb_OnSof(void);
extern void usb_OnStallSent(uint8_t ep);
extern void usb_OnError(void);
extern uint8_t usb_OnClassReq(void);
extern uint8_t usb_CheckRemoteWakeup(void);
//------------------------------------------------------------------------------
//usb_ep0.c
void ep0_hdr(void);
void ep0_next_trf(void);

struct ctrl_trf {
	uint8_t state; //stav riadiaceho prenosu, enum ctrl_trf_state
	uint8_t trunc; //priznak skratenia dlzky prenosu
	uint16_t bytes; /* realny pocet bajtov na prenos - zariadenie moze poslat
                       menej */
	uint8_t *data; //prenasane data: NULL znamena ze data su uz v buffri endp
};

extern struct ctrl_trf ctrl;
//------------------------------------------------------------------------------
enum { /* stavy usb zariadenia */
	USB_SUSPENDED,
	USB_POWERED,
	USB_DEFAULT,
	USB_ATTACHED,
	USB_ADDR_PENDING,
	USB_ADDRESSED,
	USB_CONFIGURED,
};

extern uint8_t usb_state;             /* stav usb zariadenia */
extern union usb_status_t usb_status; /* remote_wakeup, ..., GET_STATUS */
extern void (*ep_hdrs[7])(void);      /* obsluzne fcie pre endpointy */
extern uint8_t cur_cfg;               /* momentalne pouzivana konfiguracia,
                                         0 = ziadna */
#define USB_EP0_BUF_SIZE (16)
//------------------------------------------------------------------------------
//USB RAM
extern struct usb_bdt bdt          @0x1860; /* buffer descriptor table, 32B */
extern union ep0out_u ep0out_setup @0x1880;
extern unsigned char ep0in_buf[USB_EP0_BUF_SIZE]  @0x1890;
//------------------------------------------------------------------------------
//deskriptory
extern const uint8_t device_descriptor[];    //device deskriptor
extern const uint8_t *cfg_desc_array[];      //pole konfiguracnych deskriptorov
extern const uint8_t *str_descs[];           //pole stringovych deskriptorov
/* pole obsluznych funkcii udalosti konfiguracie */
extern const void (*cfg_hdrs[])(uint8_t event);

#endif
